/**************************************************************

phot_aper.h (C-Munipack project)
Table of apertures
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef PHOT_APER_H
#define PHOT_APER_H

#include "hash.h"
#include "context.h"
#include "cmpack_console.h"

/*********************   Datatypes   *******************/

/* Aperture definition */
struct _CmpackPhtAperture
{
	CmpackContext gc;				/**< Global context */
	int id;							/**< Aperture unique identifier */
	int mask;						/**< Bitmask of valid fields */
	double radius;					/**< Aperture radius in pixels */
};

/* Table of apertures */
struct _CmpackAperTab
{
	int	count;						/**< Number of apertures */
	int	capacity;					/**< Number of allocated records */
	CmpackPhtAperture **list;		/**< List of apertures */
};
typedef struct _CmpackAperTab CmpackAperTab;

/********************   Public functions    ***********************/

/* Init table */
void pht_aptab_init(CmpackAperTab *tab);
	
/* Clear table */
void pht_aptab_clear(CmpackAperTab *tab);
	
/* Copy table */
void pht_aptab_copy(CmpackAperTab *dst, const CmpackAperTab *src);
	
/* Dump table */
void pht_aptab_dump(CmpackConsole *ctx, CmpackAperTab *tab);
	
/* Write table */
void pht_aptab_write(CmpackAperTab *tab, FILE *to);
	
/* Add aperture, returns its index */
int pht_aptab_add(CmpackAperTab *tab, CmpackPhtAperture *aper);
	
/* Delete aperture */
void pht_aptab_delete(CmpackAperTab *tab, int index);
	
/* Create a new aperture */
CmpackPhtAperture *pht_aper_new(int id);

#endif
