/**************************************************************

phot_stars.h (C-Munipack project)
Table of stars
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef PHOT_STARS_H
#define PHOT_STARS_H

#include "hash.h"
#include "context.h"
#include "cmpack_console.h"
#include "cmpack_phtfile.h"

/*********************   Datatypes   *******************/

/* Measurement info */
struct _CmpackMag
{
	int aperture;					/**< Aperture identifier */
	CmpackPhtMeasurement data;		/**< Measurement */
};
typedef struct _CmpackMag CmpackMag;

/* Table of measurements */
struct _CmpackMagList
{
	int count;						/**< Number of items */
	int	capacity;					/**< Number of allocated records */
	CmpackMag *list;				/**< List of magnitudes */
};
typedef struct _CmpackMagList CmpackMagList;

/* Star info */
struct _CmpackPhtStar
{
	CmpackContext gc;				/**< Global context */
	int mask;						/**< Bitmask of valid fields */
	double center[2];				/**< Position of center */
	int id;							/**< Star's unique identifier */
	int refid;						/**< Star's reference identifier */
	CmpackMagList mag;				/**< List of measurements */
};

/* Table of stars */
struct _CmpackStarTab
{
	int count;						/**< Number of stars */
	int capacity;					/**< Number of allocated records */
	CmpackPhtStar **list;			/**< List of stars */
};
typedef struct _CmpackStarTab CmpackStarTab;

/********************   Public functions    ***********************/

/* Init the table */
void pht_stars_init(CmpackStarTab *tab);
	
/* Clear the table */
void pht_stars_clear(CmpackStarTab *tab);
	
/* Dump the table */
void pht_stars_dump(CmpackConsole *ctx, CmpackStarTab *tab);
	
/* Write all stars to file */
void pht_stars_write(CmpackStarTab *tab, FILE *to);
	
/* Copy table of stars */
void pht_stars_copy(CmpackStarTab *dsttab, const CmpackStarTab *srcttab);
	
/* Creates a new star and returns its index */
int pht_stars_add(CmpackStarTab *tab, CmpackPhtStar *star);
	
/* Delete star by its index */
void pht_stars_delete(CmpackStarTab *tab, int index);
	
/* Find star by identifier, return its index */
int pht_stars_search(CmpackStarTab *tab, int starid);
	
/* Find star by identifier, return its index */
int pht_stars_search_by_refid(CmpackStarTab *tab, int refid);
	
/* New star */
CmpackPhtStar *pht_star_new(int starid);

/* Get measurement */
CmpackPhtMeasurement *pht_star_get_data(CmpackPhtStar *star, int aperture);

/* Set measurement */
void pht_star_set_data(CmpackPhtStar *star, int aperture, const CmpackPhtMeasurement *data);

/* Clear measurements */
void pht_star_clear(CmpackPhtStar *star);

#endif
