/**************************************************************

table.h (C-Munipack project)
Tables -- private header file
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_DATA_H
#define CMPACK_DATA_H

#include "cmpack_console.h"

/***********************      Datatypes      ******************/

typedef struct _CmpackTabData CmpackTabData;

/******************   Public functions    *********************/

/* Init table data */
CmpackTabData *tab_data_create(void);
	
/* Free allocated memory */
void tab_data_free(CmpackTabData *);
	
/* Dump table data */
void tab_data_dump(CmpackTabData *data, CmpackConsole *con);
	
/* Write data to file */
void tab_data_write_cols(CmpackTabData *data, FILE *to, const char *prefix);

/* Write data to file */
void tab_data_write_data(CmpackTabData *data, FILE *to);

/* Clear data */
void tab_data_clear(CmpackTabData *data);
	
/* Dump table data */
void tab_data_copy(CmpackTabData *dst, const CmpackTabData *src); 

/* Add a new row, returns its index */
int tab_data_add_row(CmpackTabData *data);
	
/* Insert a row */
int tab_data_insert_row(CmpackTabData *data, int index);
	
/* Delete a row */
void tab_data_delete_row(CmpackTabData *data, int index);

/* Append a column */
int tab_data_add_col(CmpackTabData *data, const char *name, CmpackColumnType type, int prec);

/* Insert a column */
int tab_data_insert_col(CmpackTabData *data, int index, const char *name, CmpackColumnType type, int prec);

/* Delete a column */
void tab_data_delete_col(CmpackTabData *data, int index);

/* Get number of columns */
int tab_data_ncols(CmpackTabData *data);

/* Get number of rows */
int tab_data_nrows(CmpackTabData *data);

/* Get column by index */
CmpackTabColumn *tab_data_get_col(CmpackTabData *data, int index); 

/* Get column by name */
int tab_data_find_col(CmpackTabData *data, const char *name); 

/* Get column by name */
const char *tab_cols_name(CmpackTabData *data, int index);

/* Get column by name */
const char *tab_col_name(CmpackTabColumn *col);

/* Get column by name */
void tab_col_setname(CmpackTabColumn *col, const char *name);

/* Get column by name */
CmpackColumnType tab_col_type(CmpackTabColumn *col);

/* Set individual table data */
void tab_data_set(CmpackTabData *data, int col, int row, const char *value);
void tab_data_seti(CmpackTabData *data, int col, int row, int value);
void tab_data_setd(CmpackTabData *data, int col, int row, double value);

/* Clear individual table data */
void tab_data_unset(CmpackTabData *data, int col, int row);

/* Get individual table data */
int tab_data_get(CmpackTabData *data, int col, int row, char **value);
int tab_data_geti(CmpackTabData *data, int col, int row, int *value);
int tab_data_getd(CmpackTabData *data, int col, int row, double *value);

#endif
