/**************************************************************

file_dlg.h (C-Munipack project)
The preview dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_CCD_FILE_DLG_H
#define CMPACK_CCD_FILE_DLG_H

#include "file_dlg.h"
#include "info_dlg.h"
#include "chart.h"
#include "chart_ex.h"
#include "menubar.h"
#include "scale.h"
#include "statusbar.h"
#include "infobox.h"
#include "qphot.h"

//
// Non-modal file viewer
//
class CCCDFileDlg:public CFileDlg
{
public:
	// Constructor
	CCCDFileDlg(GtkWindow *pParent);

protected:
	// Load a file
	virtual bool LoadFile(const char *fpath);

private:
	enum tInfoMode {
		INFO_NONE,
		INFO_PHOT
	};

	GtkWidget		*m_ZoomEntry;
	GtkToolItem		*m_ZoomIn, *m_ZoomOut;
	CMenuBar		m_Menu;
	CChartEx		m_Chart;
	CScale			m_ScaleBox;
	CStatusBar		m_Status;
	CInfoBox		m_InfoBox;
	bool			m_Updating, m_PhotReady;
	int				m_LastPosX, m_LastPosY, m_Aperture;
	tPhotResult		m_PhotResult;
	tInfoMode		m_InfoMode;

	void OnCommand(int cmd_id);
	void ShowProperties(void);
	void UpdateControls(void);
	void UpdateInfoBox(void);
	void OnButtonClicked(GtkWidget *pBtn);
	void OnZoomChanged(void);
	void OnMouseMoved(void);
	void OnLButtonClick(guint32 timestamp);

	static void button_clicked(GtkWidget *pButton, CCCDFileDlg *pDlg);

	static void MenuClick(int message, int wparam, void* lparam, void* cb_data);
	static void ChartCallback(int message, int wparam, void* lparam, void* cb_data);
};


//
// Modal file viewer
//
class CCCDFileDlgModal:public CFileDlgModal
{
public:
	// Constructor
	CCCDFileDlgModal(GtkWindow *pParent);

protected:
	// Load a file
	virtual bool LoadFile(const char *fpath);

private:
	CMenuBar		m_Menu;
	CChartEx		m_Chart;

	void Clear(void);
	void OnCommand(int cmd_id);
	void ShowProperties(void);

	static void MenuClick(int message, int wparam, void* lparam, void* cb_data);
};

//
// File properties dialog
//
class CCCDFileInfoDlg:public CInfoDlg
{
public:
	// Constructor
	CCCDFileInfoDlg(GtkWindow *pParent);

	// Destructor
	virtual ~CCCDFileInfoDlg(void);

	// Display dialog
	void ShowModal(const char *path, const CImageInfo *info);

private:
	gchar *m_Path;
	GtkWidget *m_HdrBtn;

	void OnButtonClicked(GtkWidget *pBtn);
	void ShowHeader(void);

	static void button_clicked(GtkWidget *pButton, CCCDFileInfoDlg *pDlg);
};

#endif
