/**************************************************************

filedlg_base.h (C-Munipack project)
The base class for file preview dialogs
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_FILE_DLG_H
#define CMPACK_FILE_DLG_H

#include <gtk/gtk.h>

//
// Base class for non-modal file preview windows
//
class CFileDlg
{
public:
	// Constructor
	CFileDlg(GtkWindow *pParent);

	// Destructor
	virtual ~CFileDlg();

	// Load the file
	bool Load(const char *path);

	// Show non-modal dialog
	void Show();

	// Hide non-modal dialog
	void Destroy();

	// Get frame identifier
	const char *Path(void) const
	{ return m_Path; }

protected:
	GtkWidget *m_pDlg;
	GtkWindow *m_pParent;
	char *m_Path, *m_FileName, *m_DirPath;

	// Load the file
	virtual bool LoadFile(const char *path) = 0;

	// Called before the window is presented
	virtual void OnBeforeShow(void) {}

	// Change path
	void SetPath(const char *path);

private:
	static void destroy(GtkObject *pWnd, CFileDlg *pDlg);
	static void delete_event(GtkWidget *pWnd, GdkEvent *event, CFileDlg *user_data);
};


//
// Base class for modal file preview dialogs
//
class CFileDlgModal
{
public:
	// Constructor
	CFileDlgModal(GtkWindow *pParent);

	// Destructor
	virtual ~CFileDlgModal();

	// Show modal dialog
	void ShowModal(const char *fpath);

protected:
	GtkWidget *m_pDlg;
	GtkWindow *m_pParent;
	char *m_Path, *m_FileName, *m_DirPath;

	// Load the file
	virtual bool LoadFile(const char *path) = 0;

	// Change path
	void SetPath(const char *path);

};

#endif
