/**************************************************************

starinfo.cpp (C-Munipack project)
The toolbox window that shows the information about a star
Copyright (C) 2009 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "infobox.h"

struct tFieldRec
{
	int id;
	GtkWidget *label;
	GtkWidget *value;
};

static void fieldrec_free(struct tFieldRec *rec, gpointer user_data)
{
	g_free(rec);
}

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

CInfoBox::CInfoBox():m_Visible(false), m_Labels(NULL)
{
	// Dialog frame
	m_Frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(m_Frame), GTK_SHADOW_IN);

	// Dialog layout
	m_Table = gtk_table_new(0, 0, false);
	gtk_table_set_row_spacings(GTK_TABLE(m_Table), 4);
	gtk_table_set_col_spacings(GTK_TABLE(m_Table), 4);
	gtk_container_add(GTK_CONTAINER(m_Frame), m_Table);
	gtk_container_set_border_width(GTK_CONTAINER(m_Table), 4);

}

CInfoBox::~CInfoBox()
{
	g_list_foreach(m_Labels, (GFunc)fieldrec_free, NULL);
	g_list_free(m_Labels);
}

void CInfoBox::Clear(void)
{
	GList *ptr;

	// Destroy widgets
	for (ptr=m_Labels; ptr!=NULL; ptr=ptr->next) {
		if (((tFieldRec*)ptr->data)->label)
			gtk_widget_destroy(((tFieldRec*)ptr->data)->label);
		if (((tFieldRec*)ptr->data)->value)
			gtk_widget_destroy(((tFieldRec*)ptr->data)->value);
		g_free(((tFieldRec*)ptr->data));
	}
	g_list_free(m_Labels);
	m_Labels = NULL;
	// Resize the m_Table
	gtk_table_resize(GTK_TABLE(m_Table), 1, 1);
}

void CInfoBox::AddField(int field, int col, int row, const char *caption)
{
	tFieldRec *rec = (tFieldRec*)g_malloc0(sizeof(tFieldRec));
	
	rec->id = field;
	rec->label = gtk_label_new(caption);
	gtk_misc_set_alignment(GTK_MISC(rec->label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(m_Table), rec->label, 2*col, 2*col+1, row, row+1, 
		GTK_FILL, (GtkAttachOptions)0, 4, 0);
	rec->value = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(rec->value), 0.0, 0.5);
	gtk_label_set_max_width_chars(GTK_LABEL(rec->value), 60);
	gtk_table_attach(GTK_TABLE(m_Table), rec->value, 2*col+1, 2*col+2, row, row+1,
		(GtkAttachOptions)(GTK_FILL | GTK_EXPAND), (GtkAttachOptions)0, 4, 0);
	m_Labels = g_list_append(m_Labels, rec);
}

void CInfoBox::AddHeading(int id, int col, int row, const char *caption, const char *color)
{
	char buf[128];
	tFieldRec *rec = (tFieldRec*)g_malloc0(sizeof(tFieldRec));

	rec->id = id;
	rec->label = gtk_label_new(NULL);
	gtk_misc_set_alignment(GTK_MISC(rec->label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(m_Table), rec->label, 2*col, 2*col+2, row, row+1, 
		GTK_FILL, (GtkAttachOptions)0, 4, 0);
	if (caption) {
		if (color) {
			sprintf(buf, "<span weight=\"bold\" color=\"%s\" underline=\"single\">%s</span>", color, caption);
			gtk_label_set_markup(GTK_LABEL(rec->label), buf);
		} else {
			sprintf(buf, "<span weight=\"bold\" underline=\"single\">%s</span>", caption);
			gtk_label_set_markup(GTK_LABEL(rec->label), buf);
		}
	}
	m_Labels = g_list_append(m_Labels, rec);
}

void CInfoBox::Show(bool show)
{
	if (show) {
		gtk_widget_show_all(m_Frame);
		m_Visible = true;
	} else {
		gtk_widget_hide(m_Frame);
		m_Visible = false;
	}
}

void CInfoBox::SetHeading(int id, const char *caption, const char *color)
{
	char buf[128];
	GList *ptr;

	for (ptr=m_Labels; ptr!=NULL; ptr=ptr->next) {
		if (((tFieldRec*)(ptr->data))->id == id) {
			if (color) {
				sprintf(buf, "<span weight=\"bold\" color=\"%s\" underline=\"single\">%s</span>", color, caption);
				gtk_label_set_markup(GTK_LABEL(((tFieldRec*)(ptr->data))->label), buf);
			} else {
				sprintf(buf, "<span weight=\"bold\" underline=\"single\">%s</span>", caption);
				gtk_label_set_markup(GTK_LABEL(((tFieldRec*)(ptr->data))->label), buf);
			}
			break;
		}
	}
}

void CInfoBox::SetField(int id, const char *value)
{
	GList *ptr;

	for (ptr=m_Labels; ptr!=NULL; ptr=ptr->next) {
		if (((tFieldRec*)(ptr->data))->id == id) {
			if (value)
				gtk_label_set_label(GTK_LABEL(((tFieldRec*)(ptr->data))->value), value);
			else
				gtk_label_set_label(GTK_LABEL(((tFieldRec*)(ptr->data))->value), "");
			break;
		}
	}
}

void CInfoBox::SetField(int id, int value, const char *unit)
{
	GList *ptr;
	char buf[128];

	for (ptr=m_Labels; ptr!=NULL; ptr=ptr->next) {
		if (((tFieldRec*)(ptr->data))->id == id) {
			if (unit)
				sprintf(buf, "%d %s", value, unit);
			else
				sprintf(buf, "%d", value);
			gtk_label_set_label(GTK_LABEL(((tFieldRec*)(ptr->data))->value), buf);
			break;
		}
	}
}

void CInfoBox::SetField(int id, double value, int prec, const char *unit)
{
	GList *ptr;
	char buf[128];

	for (ptr=m_Labels; ptr!=NULL; ptr=ptr->next) {
		if (((tFieldRec*)(ptr->data))->id == id) {
			if (unit)
				sprintf(buf, "%.*f %s", prec, value, unit);
			else
				sprintf(buf, "%.*f", prec, value);
			gtk_label_set_label(GTK_LABEL(((tFieldRec*)(ptr->data))->value), buf);
			break;
		}
	}
}
