/**************************************************************

preview.h (C-Munipack project)
Preview widget
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef _PREVIEW_H
#define _PREVIEW_H

#include <gtk/gtk.h>

#include "callback.h"
#include "chart.h"

class CPreview:public CCBObject
{
public:
	// Constructor
	CPreview(void);

	// Destructor
	virtual ~CPreview();

	// Get widget
	GtkWidget *Handle(void)
	{ return m_Widget; }

	// Set parent window
	void SetParent(GtkWindow *parent);

	// Clear preview
	void Clear(void);

	// Show file
	void Update(const char *filepath);

	// Set file type mask
	void SetTypeMask(int bitmask);

protected:
	char			*m_Path;					// File path
	GtkWindow		*m_pParent;					// Parent window
	int				m_TypeMask;					// Mask of file types
	GtkWidget		*m_Widget;					// Whole widget
	GtkWidget		*m_Info;					// Info box
	GtkWidget		*m_DrawArea;				// Drawing area
	GdkPixmap		*m_Pixmap;					// Backscreen buffer
	int				m_PixWidth, m_PixHeight;	// Backscreen buffer size
	double			m_RefMag;					// Reference magnitude
	int				m_ApertureId;				// Default aperture id
	cairo_surface_t	*m_Image;					// Image
	int				m_ImgWidth, m_ImgHeight;	// Image size
	struct tStarRec	*m_Stars;					// List of stars
	int				m_StCount;					// Number of stars, allocated space
	int				m_PhtWidth, m_PhtHeight;	// Chart size
	struct tDataRec *m_Data;					// Table data
	int				m_DataCount;				// Size of table (number of rows)
	double			m_MinX, m_MaxX;				// X axis data range
	double			m_MinY, m_MaxY;				// Y axis data range
	char			*m_Text;					// Displayed text

	// Create a new backing pixmap of the appropriate size 
	static gint configure_event(GtkWidget *widget, GdkEventConfigure *event, CPreview *pMe);
	void OnConfigure(GtkWidget *widget, GdkEventConfigure *event);

	// Redraw the screen from the backing pixmap 
	static gint expose_event(GtkWidget *widget, GdkEventExpose *event, CPreview *pMe);
	void OnExposeEvent(GtkWidget *widget, GdkEventExpose *event);

	// Click handling
	static gint button_press_event(GtkWidget *widget, GdkEventButton *event, CPreview *pMe);
	void OnButtonPress(GtkWidget *widget, GdkEventButton *event);

	// Set file path
	void SetPath(const char *path);

	// Load image file
	void LoadImage(const char *path);

	// Load image data
	void UpdateImageData(CmpackCcdFile *file);

	// Load image info
	void UpdateImageInfo(CmpackCcdFile *file);

	// Load photometry file
	void LoadChart(const char *path);

	// Load photometry file data
	void UpdateChartData(CmpackPhtFile *file);

	// Load photometry file info
	void UpdateChartInfo(CmpackPhtFile *file);

	// Load catalogue file
	void LoadCatalog(const char *path);

	// Load catalogue file data
	void UpdateCatalogData(CmpackCatFile *file);

	// Load catalogue file info
	void UpdateCatalogInfo(CmpackCatFile *file);

	// Load catalogue file
	void LoadTable(const char *path);

	// Load catalogue file data
	void UpdateTableData(CmpackTable *table);

	// Load catalogue file info
	void UpdateTableInfo(CmpackTable *table);

	// Load readall file
	void LoadReadall(const char *path);

	// Update back-screen buffer
	void Redraw(void);

	// Clear data
	void ClearData(void);

	// On show file
	void ShowFileModal(void);
};

#endif
