/**************************************************************

scale.h (C-Munipack project)
Widget which can draw a scale
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef MUNIWIN_SCALE_H
#define MUNIWIN_SCALE_H

#include <gtk/gtk.h>
#include <cmunipack.h>

#include "callback.h"
#include "project.h"

class CScale
{
public:
	CScale();

	virtual ~CScale();

	// Get handle to the drawing area
	GtkWidget *Handle(void)
	{ return m_Frame; }

	// Show/hide widget
	void Show(bool show);

	// Visible?
	bool Visible(void) const
	{ return m_Visible; }

	// Set chart
	void SetChart(class CChartBase *pChart);

	// Update scale
	void Update(void);

private:
	bool				m_Visible;
	int					m_PixWidth, m_PixHeight;
	GtkWidget			*m_Frame;				// Scrolled window
	GtkWidget			*m_DrawArea;			// Drawing area
	GdkPixmap			*m_Pixmap;				// Backscreen buffer
	cairo_surface_t		*m_Image;				// Background image
	CChartBase			*m_pChart;				// Reference to chart

	// Create a new backing pixmap of the appropriate size 
	static gint configure_event(GtkWidget *widget, GdkEventConfigure *event, CScale *pMe);
	void OnConfigure(GtkWidget *widget, GdkEventConfigure *event);

	// Redraw the screen from the backing pixmap 
	static gint expose_event(GtkWidget *widget, GdkEventExpose *event, CScale *pMe);
	void OnExpose(GtkWidget *widget, GdkEventExpose *event);

	// Update back-screen buffer
	void draw_brush(GtkWidget *widget, gdouble x, gdouble y);

	// Rebuild image
	void RebuildImage(void);

	// Update back-screen buffer
	void Redraw(void);
};

#endif
