/**************************************************************

stars_dlg.cpp (C-Munipack project)
The 'Choose stars' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "project.h"
#include "chart.h"
#include "stars_dlg.h"
#include "main.h"
#include "utils.h"
#include "configuration.h"

//-------------------------   POPUP MENU   ---------------------------

enum tPopupCommand
{
	CMD_SET_VARIABLE,
	CMD_SET_COMPARISON,
	CMD_SET_CHECK,
	CMD_UNSET
};

static const CPopupMenu::tPopupMenuItem SelectMenu[] = {
	{ CPopupMenu::MB_ITEM, CMD_SET_VARIABLE,	"_Variable" },
	{ CPopupMenu::MB_ITEM, CMD_SET_COMPARISON, "_Comparison" },
	{ CPopupMenu::MB_ITEM, CMD_SET_CHECK,		"Chec_k" },
	{ CPopupMenu::MB_SEPARATOR },
	{ CPopupMenu::MB_ITEM, CMD_UNSET,			"_Unselect" },
	{ CPopupMenu::MB_END }
};

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

CStarsDlg::CStarsDlg(GtkWindow *pParent):m_LastIndex(-1)
{
	GSList *group;
	GtkWidget *hbox, *frame, *fbox, *vbox;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Choose stars", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Dialog icon
	gchar *icon = get_icon_file("stars");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Preview box
	hbox = gtk_hbox_new(FALSE, 8);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), GTK_WIDGET(hbox), TRUE, TRUE, 0);

	// Chart
	frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(frame), GTK_WIDGET(g_object_ref(m_Chart.Handle())));
	gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(frame), TRUE, TRUE, 0);
	m_Chart.EnableTracking(true);
	m_Chart.RegisterCallback(ChartClick, this);
	gtk_widget_set_size_request(m_Chart.Handle(), 640, 480);

	// Sidebar
	vbox = gtk_vbox_new(FALSE, 8);
	gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(vbox), FALSE, FALSE, 0);
	m_ClearBtn = gtk_button_new_with_label("Clear");
	gtk_box_pack_start(GTK_BOX(vbox), m_ClearBtn, FALSE, TRUE, 0);
	m_RestoreBtn = gtk_button_new_with_label("Restore");
	gtk_box_pack_start(GTK_BOX(vbox), m_RestoreBtn, FALSE, TRUE, 0);
	frame = gtk_frame_new("Show");
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	fbox = gtk_vbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(fbox), 4);
	gtk_container_add(GTK_CONTAINER(frame), fbox);
	m_ShowImage = gtk_radio_button_new_with_label(NULL, "Image only");
	gtk_box_pack_start(GTK_BOX(fbox), m_ShowImage, FALSE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_ShowImage));
	m_ShowChart = gtk_radio_button_new_with_label(group, "Chart only");
	gtk_box_pack_start(GTK_BOX(fbox), m_ShowChart, FALSE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(m_ShowImage));
	m_ShowFull = gtk_radio_button_new_with_label(group, "Image + chart");
	gtk_box_pack_start(GTK_BOX(fbox), m_ShowFull, FALSE, TRUE, 0);

	// Make popup menus
	m_SelectMenu.Create(SelectMenu);
	m_SelectMenu.RegisterCallback(MenuClick, this);

	// Set initial display mode
	SetDisplayMode((tDispMode)CConfig::GetInt("StarsDlg", "Mode", DISP_FULL));
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_ShowImage), m_DispMode==DISP_IMAGE);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_ShowChart), m_DispMode==DISP_CHART);
	gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(m_ShowFull), m_DispMode==DISP_FULL);

	// Signals
	g_signal_connect(G_OBJECT(m_ShowImage), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_ShowChart), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_ShowFull), "toggled", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_ClearBtn), "clicked", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_RestoreBtn), "clicked", G_CALLBACK(button_clicked), this);

	gtk_widget_show_all(hbox);
}

CStarsDlg::~CStarsDlg()
{
	gtk_widget_destroy(m_pDlg);
}

bool CStarsDlg::Execute()
{
	int frame;
	GtkWindow *pMainWnd = Global.pMainWnd->GetHandle();
	char *ppath, *tpath;
	const char *cat_file, *tmp_file;

	// Load reference file
	m_Chart.Clear();
	Global.pProject->GetReference(&frame, &cat_file);
	if (frame>0) {
		ppath = Global.pProject->GetPhotFile(frame);
		m_Chart.LoadChart(ppath, true);
		tpath = Global.pProject->GetImageFile(frame);
		m_Chart.LoadImage(tpath);
		g_free(tpath);
		g_free(ppath);
	} else if (cat_file) {
		tmp_file = Global.pProject->GetTempCatFile();
		m_Chart.LoadChart(tmp_file, true);
		tpath = SetFileExtension(tmp_file, "fts");
		m_Chart.LoadImage(tpath);
		g_free(tpath);
	} else {
		ShowError(pMainWnd, "Please, execute the matching procedure first.");
		return false;
	}

	// Set aperture
	m_Chart.SetAperture(Global.pProject->Aperture());

	// Set initial selection
	m_Chart.SetSelection(Global.pProject->Selection());
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT) {
		// Save selection
		Global.pProject->SetSelection(m_Chart.GetSelection());
		Global.pProject->Save();
		return true;
	}
	return false;
}

void CStarsDlg::ChartClick(int message, int wparam, void *lparam, void *cb_data)
{
	CStarsDlg *pMe = (CStarsDlg*)cb_data;
		
	switch (message)
	{
	case CChartBase::CB_MOUSE_MOVE:
		pMe->OnMouseMove();
		break;
	case CChartBase::CB_LEFT_CLICK:
		pMe->OnLButtonClick((GdkEventButton*)lparam);
		break;
	}
}


void CStarsDlg::OnMouseMove(void)
{
	int index, x, y;

	if (m_Chart.GetCursorPos(&x, &y)) 
		index = m_Chart.FindStarByPos(x, y, true);
	else
		index = -1;
	if (index>=0) {
		if (m_LastIndex!=index) {
			m_LastIndex = index;
			m_Chart.SetFocusedStar(index);
		}
	} else {
		if (m_LastIndex>=0) {
			m_LastIndex = -1;
			m_Chart.SetFocusedStar(-1);
		}
	}
}


void CStarsDlg::OnLButtonClick(const GdkEventButton *event)
{
	CChartBase::tStarInfo star;

	if (m_Chart.GetStar(m_Chart.FocusedStar(), &star)) {
		m_StarId = star.id;
		m_SelectMenu.Enable(CMD_SET_VARIABLE, star.sel_type!=ST_VARIABLE);
		m_SelectMenu.Enable(CMD_SET_COMPARISON, star.sel_type!=ST_COMPARISON);
		m_SelectMenu.Enable(CMD_SET_CHECK, star.sel_type!=ST_CHECK);
		m_SelectMenu.Enable(CMD_UNSET, star.sel_type!=ST_NOT_SELECTED);
		m_SelectMenu.Popup(event);
	}
}

void CStarsDlg::MenuClick(int message, int wparam, void *lparam, void *cb_data)
{
	CStarsDlg *pMe = (CStarsDlg*)cb_data;
		
	switch (message)
	{
	case CPopupMenu::CB_ACTIVATE:
		switch (wparam)
		{
		case CMD_SET_VARIABLE:
			pMe->m_Chart.GetSelection()->Select(pMe->m_StarId, ST_VARIABLE);
			break;
		case CMD_SET_COMPARISON:
			pMe->m_Chart.GetSelection()->Select(pMe->m_StarId, ST_COMPARISON);
			break;
		case CMD_SET_CHECK:
			pMe->m_Chart.GetSelection()->Select(pMe->m_StarId, ST_CHECK);
			break;
		case CMD_UNSET:
			pMe->m_Chart.GetSelection()->Unselect(pMe->m_StarId);
			break;
		}
	}
}

void CStarsDlg::button_clicked(GtkWidget *button, CStarsDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CStarsDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn == m_ShowImage) 
		SetDisplayMode(DISP_IMAGE);
	else if (pBtn == m_ShowChart) 
		SetDisplayMode(DISP_CHART);
	else if (pBtn == m_ShowFull) 
		SetDisplayMode(DISP_FULL);
	else if (pBtn == m_ClearBtn)
		m_Chart.GetSelection()->Clear();
	else if (pBtn == m_RestoreBtn)
		RestoreSelection();
}

void CStarsDlg::SetDisplayMode(tDispMode mode)
{
	m_DispMode = mode;
	CConfig::SetInt("StarsDlg", "Mode", m_DispMode);

	m_Chart.ShowImage(m_DispMode!=DISP_CHART);
	m_Chart.ShowChart(m_DispMode!=DISP_IMAGE);
	m_Chart.ShowBadStars(false);
	m_Chart.ShowNonMatchedStars(false);
}

void CStarsDlg::RestoreSelection(void)
{
	int i;
	GtkWidget *pDlg, *frame, *fbox, *last, *catfile, *varfind;
	GSList *group;
	
	if (Global.pProject->Selection()->Count()==0 &&
		Global.pProject->CatFileSelection()->Count()==0 &&
		Global.pProject->VarFindSelection()->Count()==0)
		return;

	if (Global.pProject->Selection()->Count()>0 &&
		Global.pProject->CatFileSelection()->Count()==0 &&
		Global.pProject->VarFindSelection()->Count()==0) {
			m_Chart.SetSelection(Global.pProject->Selection());
			return;
	}
	if (Global.pProject->Selection()->Count()==0 &&
		Global.pProject->CatFileSelection()->Count()>0 &&
		Global.pProject->VarFindSelection()->Count()==0) {
			m_Chart.SetSelection(Global.pProject->CatFileSelection());
			return;
	}
	if (Global.pProject->Selection()->Count()==0 &&
		Global.pProject->CatFileSelection()->Count()==0 &&
		Global.pProject->VarFindSelection()->Count()>0) {
			m_Chart.SetSelection(Global.pProject->VarFindSelection());
			return;
	}

	// Dialog with buttons
	pDlg = gtk_dialog_new_with_buttons("Restore selection", GTK_WINDOW(m_pDlg), 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);

	// Sidebar
	frame = gtk_frame_new("Show");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(pDlg)->vbox), frame, FALSE, FALSE, 0);
	fbox = gtk_vbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(fbox), 4);
	gtk_container_add(GTK_CONTAINER(frame), fbox);
	last = gtk_radio_button_new_with_label(NULL, "last selection used");
	gtk_widget_set_sensitive(last, Global.pProject->Selection()->Count()>0);
	gtk_box_pack_start(GTK_BOX(fbox), last, FALSE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(last));
	catfile = gtk_radio_button_new_with_label(group, "stored in the catalogue file");
	gtk_widget_set_sensitive(catfile, Global.pProject->CatFileSelection()->Count()>0);
	gtk_box_pack_start(GTK_BOX(fbox), catfile, FALSE, TRUE, 0);
	group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(last));
	varfind = gtk_radio_button_new_with_label(group, "result of 'Find variables' tool");
	gtk_widget_set_sensitive(varfind, Global.pProject->VarFindSelection()->Count()>0);
	gtk_box_pack_start(GTK_BOX(fbox), varfind, FALSE, TRUE, 0);

	i = CConfig::GetInt("StarsDlg", "RestoreMode", 0);
	if (i==0) {
		if (Global.pProject->Selection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(last), true);
		else if (Global.pProject->CatFileSelection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(catfile), true);
		else if (Global.pProject->VarFindSelection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(varfind), true);
	} else
	if (i==1) {
		if (Global.pProject->CatFileSelection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(catfile), true);
		else if (Global.pProject->Selection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(last), true);
		else if (Global.pProject->VarFindSelection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(varfind), true);
	} else {
		if (Global.pProject->VarFindSelection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(varfind), true);
		else if (Global.pProject->Selection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(last), true);
		else if (Global.pProject->CatFileSelection()->Count()>0)
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(catfile), true);
	}

	gtk_widget_show_all(frame);

	if (gtk_dialog_run(GTK_DIALOG(pDlg))==GTK_RESPONSE_ACCEPT) {
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(last))) {
			CConfig::GetInt("StarsDlg", "RestoreMode", 0);
			m_Chart.SetSelection(Global.pProject->Selection());
		} else
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(catfile))) {
			CConfig::GetInt("StarsDlg", "RestoreMode", 1);
			m_Chart.SetSelection(Global.pProject->CatFileSelection());
		} else
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(varfind))) {
			CConfig::GetInt("StarsDlg", "RestoreMode", 2);
			m_Chart.SetSelection(Global.pProject->VarFindSelection());
		}
	}

	gtk_widget_destroy(pDlg);
}
