/**************************************************************

table_dlg.h (C-Munipack project)
The preview dialog for tables
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef CMPACK_TABLE_DLG_H
#define CMPACK_TABLE_DLG_H

#include "file_dlg.h"
#include "info_dlg.h"
#include "graph.h"
#include "graph_data.h"
#include "menubar.h"
#include "scale.h"
#include "statusbar.h"
#include "infobox.h"
#include "helper_classes.h"

//
// Non-modal file viewer
//
class CTableDlg:public CFileDlg
{
public:
	// Constructor
	CTableDlg(GtkWindow *pParent);

protected:
	// Load a file
	virtual bool LoadFile(const char *fpath);

private:
	CMenuBar		m_Menu;
	CGraph			m_Graph;
	CGraphData		m_Data;
	CTableInfo		m_Info;
	CStatusBar		m_Status;
	bool			m_Updating, m_Errors;
	int				m_Column;
	int				m_LastPosX, m_LastPosY;
	int				m_LastIndex;
	GtkWidget		*m_DataView, *m_ShowErrors;
	GtkListStore	*m_DataList;

	void LoadLightCurve(CmpackTable *table);
	void LoadTrackList(CmpackTable *table);
	void LoadMuniFind(CmpackTable *table);
	void LoadAirMass(CmpackTable *table);
	void SetData(int column);

	void OnCommand(int cmd_id);
	void ShowProperties(void);
	void UpdateControls(void);
	void OnZoomChanged(void);
	void OnMouseMoved(void);
	void OnLButtonClick(guint32 timestamp);
	void OnSelectionChanged(GtkTreeSelection *selection);

	static void MenuCallback(int message, int wparam, void* lparam, void* cb_data);
	static void GraphCallback(int message, int wparam, void* lparam, void* cb_data);
	static void selection_changed(GtkTreeSelection *widget, CTableDlg *pMe);
	static void col_name_free(char *str, void *data);
};


//
// Modal file viewer
//
class CTableDlgModal:public CFileDlgModal
{
public:
	// Constructor
	CTableDlgModal(GtkWindow *pParent);

protected:
	// Load a file
	virtual bool LoadFile(const char *fpath);

private:
	CMenuBar		m_Menu;
	CGraph			m_Graph;
	CGraphData		m_Data;
	CTableInfo		m_Info;

	void Clear(void);
	void OnCommand(int cmd_id);
	void ShowProperties(void);

	static void MenuClick(int message, int wparam, void* lparam, void* cb_data);
};

//
// File properties dialog
//
class CTableInfoDlg:public CInfoDlg
{
public:
	// Constructor
	CTableInfoDlg(GtkWindow *pParent);

	// Destructor
	virtual ~CTableInfoDlg(void);

	// Display dialog
	void ShowModal(const char *path, const CTableInfo *data);

private:
	gchar *m_Path;
	GtkWidget *m_HdrBtn;

	void OnButtonClicked(GtkWidget *pBtn);
	void ShowHeader(void);

	static void button_clicked(GtkWidget *pButton, CTableInfoDlg *pDlg);
};

#endif
