/**************************************************************

toolbar.cpp (C-Munipack project)
Tool bar wrapper
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <glib.h>

#include "utils.h"
#include "toolbar.h"

struct tToolBtnData
{
	CToolBar *tbar;
	GtkWidget *btn;
	int cmd_id;
};

// Default constructor
CToolBar::CToolBar():m_Count(0), m_hBar(NULL), m_Data(NULL)
{
}

// Destructor
CToolBar::~CToolBar()
{
	if (m_Data)
		g_free(m_Data);
}

// Create and initialize the m_hBar
void CToolBar::Create(const tToolBtn *btns)
{
	int i, count;
	char *icon;
	
	m_hBar = gtk_toolbar_new();
	gtk_toolbar_set_orientation (GTK_TOOLBAR (m_hBar), GTK_ORIENTATION_HORIZONTAL);
	gtk_toolbar_set_style (GTK_TOOLBAR (m_hBar), GTK_TOOLBAR_ICONS);

	if (btns) {
		count = 0;
		for (i=0; btns[i].type!=TB_END; i++)
			count++;
		if (count>0) {
			m_Data = (tToolBtnData*) g_realloc(m_Data, count*sizeof(tToolBtnData));
		}
		for (i=0; btns[i].type!=TB_END; i++) {
			switch (btns[i].type)
			{
			case TB_PUSHBUTTON:
				icon = get_icon_file(btns[i].icon);
				m_Data[i].btn = gtk_toolbar_append_element(GTK_TOOLBAR (m_hBar), 
					GTK_TOOLBAR_CHILD_BUTTON, NULL, NULL, btns[i].ttip, "Private", 
					gtk_image_new_from_file(icon), GTK_SIGNAL_FUNC(push_event), &m_Data[i]);
				g_free(icon);
				m_Data[i].tbar = this;
				m_Data[i].cmd_id = btns[i].cmd_id;
				break;

			case TB_SEPARATOR:
				gtk_toolbar_append_space(GTK_TOOLBAR(m_hBar));
				break;

			default:
				break;
			}
		}
		m_Count = count;
	}
	gtk_toolbar_set_tooltips (GTK_TOOLBAR (m_hBar), true);
	gtk_widget_show (m_hBar);
}

void CToolBar::push_event(GtkWidget *widget, gpointer data)
{
	((tToolBtnData*)data)->tbar->OnPushEvent(((tToolBtnData*)data)->cmd_id);
}

void CToolBar::OnPushEvent(int cmd_id)
{
	Callback(CB_ACTIVATE, cmd_id);
}

void CToolBar::Show(int cmd_id, bool show)
{
	int i;

	for (i=0; i<m_Count; i++) {
		if (m_Data[i].cmd_id == cmd_id) {
			if (show)
				gtk_widget_show(GTK_WIDGET(m_Data[i].btn));
			else
				gtk_widget_hide(GTK_WIDGET(m_Data[i].btn));
			return;
		}
	}
}

void CToolBar::Enable(int cmd_id, bool enable)
{
	int i;

	for (i=0; i<m_Count; i++) {
		if (m_Data[i].cmd_id == cmd_id) {
			gtk_widget_set_sensitive(GTK_WIDGET(m_Data[i].btn), enable);
			return;
		}
	}
}
